#include QMK_KEYBOARD_H
#include "mousekey.h"

#define _BASE 0
#define _RAISE 1
#define _LOWER 2
#define _MOUSE 3
#define _MOUSE2 4
// Fillers to make layering more clear

#define ____ KC_TRNS

#define SFT_ESC  SFT_T(KC_ESC)
#define CTL_BSPC CTL_T(KC_BSPC)
#define ALT_SPC  ALT_T(KC_SPC)
#define SFT_ENT  SFT_T(KC_ENT)

#define KC_ML KC_MS_LEFT
#define KC_MR KC_MS_RIGHT
#define KC_MU KC_MS_UP
#define KC_MD KC_MS_DOWN
#define KC_MB1 KC_MS_BTN1
#define KC_MB2 KC_MS_BTN1

#define RAISE MO(_RAISE)
#define LOWER MO(_LOWER)
#define MOUSE MO(_MOUSE)
#define MOUSE2 MO(_MOUSE2)

#define ENTRISE LT(_RAISE, KC_ENT)
#define TABRISE LT(_RAISE, KC_TAB)
#define CTLESC LCTL_T(KC_ESC)
#define CTLF6 RCTL_T(KC_F6)

/* Macros */
enum {
  NONE = 0,
  // Diagonal mouse movement
  A_MLUL,  //left up left
  A_MUL,   //up left
  A_MULU,  //up left up
  A_MURU,  //up right up
  A_MUR,   //up right
  A_MRUR,  //right up right
  A_MLDL,  //left down left
  A_MDL,
  A_MDLD,
  A_MDRD,
  A_MDR,
  A_MRDR,
  A_MTLC,  //top left corner of screen
  A_MTRC,  //top right corner
  A_MBLC,  //bottom left corner
  A_MBRC,  //bottom right corner
  A_MR1C1,  //row 1,  col 1
  A_MR1C2,  //row 1,  col 2
  A_MR1C3,
  A_MR1C4,
  A_MLE,  // left end of screen
  A_MR2C1,
  A_MR2C2,
  A_MR2C3,
  A_MR2C4,
  A_MRE,  //right end of screen
  A_MR3C1,
  A_MR3C2,
  A_MR3C3,
  A_MR3C4,
};



const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

/* Base (dvorak)
 *        +----------------------------------+                             +----------------------------------+
 *        |  '      ,      .      p       y                                   f       g     c     r       l
 * |------+------+------+------+------+------|                             |------+------+------+------+------+------|
 * | `    |   a     o     e      u      i                                     d       h     t      n      s      -
 * |------+------+------+------+------+------|                             |------+------+------+------+------+------|
 *        |  ;      q      j     k      x                                     b       m     w      v     z
 *        +------+------+------+-------------+                             +-------------+------+------+------+
 *               |     |     |                                                         |      |      |
 *               +-------------+-------------+                             +-------------+-------------+
 *                             |      |      |                             |      |      |
 *                             |------+------|                             |------+------|
 *                             |      |      |                             |      |      |
 *                             +-------------+                             +-------------+
 *                                           +-------------+ +-------------+
 *                                           |      |      | |      |      |
 *                                           |------+------| |------+------|
 *                                           |      |      | |      |      |
 *                                           +-------------+ +-------------+
 */


[_BASE] = LAYOUT( \
    _______, KC_QUOT,KC_COMM, KC_DOT, KC_P, KC_Y,            KC_F,  KC_G,  KC_C,  KC_R,  KC_L, _______,  \
    KC_GRV, KC_A,   KC_O,  KC_E,   KC_U,   KC_I,            KC_D,  KC_H,  KC_T,  KC_N,  KC_S, KC_MINS,  \
    _______,KC_SCLN, KC_Q,  KC_J,   KC_K,   KC_X,            KC_B,  KC_M,  KC_W,  KC_V,  KC_Z, _______,  \
                            KC_LSFT, ENTRISE,                          KC_RSFT,  TABRISE,  \
                                    KC_BSPC,  MOUSE,         MOUSE2, KC_SPC,         \
                                     CTLESC,  KC_LALT,          KC_RALT, CTLF6,         \
                                     KC_NO, LOWER,             LOWER, LOWER           \
),                                 /*↑このKC_NOを _______,にすると、なぜかコンパイルできない。*/


//  @ # ! %           & *
//                 ← ↓ ↑ → \ /
//     { [            ] } + =
[_LOWER] = LAYOUT(
    _______,RESET,  KC_AT, KC_HASH, KC_EXLM,KC_PERC,           _______,KC_AMPR,KC_ASTR,KC_INS, _______,_______,  \
    KC_PGUP,_______,KC_HOME,_______,KC_END ,_______,           KC_LEFT,KC_DOWN, KC_UP, KC_RGHT,KC_BSLASH,KC_SLSH, \
    _______,_______,_______,KC_LCBR,KC_LBRC,_______,           _______,KC_RBRC,KC_RCBR,KC_PLUS,KC_EQL, _______, \
                                _______,_______,                         _______,_______,                        \
                                          KC_DEL, _______,    _______,_______,                               \
                                            KC_LGUI, _______,   _______,KC_RGUI,                                 \
                                            _______,_______,    _______,_______                                \
),

//       ^ (   ) $

[_RAISE] = LAYOUT(
     _______, KC_F1,  KC_F2,  KC_F3,  KC_F4 , KC_F5,         KC_F6,  KC_F7, KC_F8, KC_F9, KC_F10,_______,  \
    KC_PGDN,KC_CIRC,KC_LPRN,_______,KC_RPRN,KC_DLR,          KC_0,   KC_1,  KC_2,  KC_3,  KC_4,  KC_PIPE,  \
    _______,KC_F11, KC_F12, KC__VOLUP,KC__VOLDOWN,_______,          KC_5,   KC_6,  KC_7,  KC_8,  KC_9,  _______,  \
                                  _______,_______,                    _______,_______,                      \
                                            _______,_______,    _______,_______,                            \
                                            _______,_______,    _______,_______,                            \
                                            _______,_______,    _______,_______                             \
     ),

[_MOUSE] = LAYOUT(
    _______,_______, M(A_MLUL),M(A_MULU),M(A_MULU),KC_NO,          KC_NO,M(A_MURU),M(A_MUR),M(A_MRUR),KC_NO ,KC_NO, \
    _______,KC_NO,   KC_MS_L, M(A_MUL),KC_MS_U,KC_WH_U,         KC_WH_D,KC_MS_D,M(A_MDR),KC_MS_R,  KC_NO, KC_NO, \
    _______,_______, M(A_MLDL),M(A_MDL),M(A_MDLD),KC_WH_L,        KC_WH_R,M(A_MDRD),M(A_MDRD),M(A_MRDR),KC_NO, KC_NO, \
                                    KC_MS_BTN2,KC_MS_BTN1,          KC_MS_BTN2, KC_MS_BTN1,                      \
                                            KC_MS_BTN3,_______,    _______,KC_MS_BTN3,                          \
                                            _______,_______,    _______,_______,                                \
                                            _______,_______,    _______,_______                                 \
     ),
[_MOUSE2] = LAYOUT(
    _______,_______, M(A_MTLC),M(A_MR1C1),M(A_MR1C2),KC_NO,       KC_NO,M(A_MR1C3),M(A_MR1C4),M(A_MTRC),KC_NO ,KC_NO, \
    _______,KC_MS_L,M(A_MLE),M(A_MR2C1),M(A_MR2C2),KC_WH_U,       KC_WH_D,M(A_MR2C3),M(A_MR2C4),M(A_MRE),KC_MS_R,KC_NO, \
    _______,_______,M(A_MBLC),M(A_MR3C1),M(A_MR3C2),KC_WH_L,      KC_WH_R,M(A_MR3C3),M(A_MR3C4),M(A_MBRC),KC_NO, KC_NO, \
                                     KC_MS_BTN2,KC_MS_BTN1,         KC_MS_BTN2, KC_MS_BTN1,                      \
                                            KC_MS_BTN3,_______,    _______,KC_MS_BTN3,                          \
                                            _______,_______,    _______,_______,                                \
                                            _______,_______,    _______,_______                                 \
     )
};

//static report_mouse_t mouse_report = {0};

// Can not drug
void mouse_move(int8_t x, int8_t y){
     report_mouse_t report = {0};
     report.x = x;
     report.y = y;
     host_mouse_send(&report);
     //mousekey_send();
}

void mouse_jump(int8_t x, int8_t y, int repeat){
     int i;
     for (i = 0; i < repeat; i++) {
          mouse_move(x, y);
     }
}


// Vertical, horizontal mouse movement
void mousemove_vh(keyrecord_t *record, int direction,  int repeat){

     if (record->event.pressed) {
          mousekey_on(direction);
        } else {
          mousekey_off(direction);
        }
     int i;
     for(i = 0; i < repeat; i++) {
        mousekey_send();
     }
     //mousekey_clear();
}

// Diagonal mouse movement
void mousemove_dgnl(keyrecord_t *record, int direction1, int direction2,  int repeat){

     if (record->event.pressed) {
          mousekey_on(direction1);
          mousekey_on(direction2);
        } else {
          mousekey_off(direction1);
          mousekey_off(direction2);
        }
     int i;
     for(i = 0; i < repeat; i++) {
        mousekey_send();
     }
     //mousekey_clear();
}

static const int8_t w = 127; //  width
static const int8_t h = 90; //  height

void mouse2top_left_corner(keyrecord_t *record) {
     mouse_jump(-w, -h, 5);
     tap_code(KC_ESC);  //prevent GNOME  ACTIVITY activate
}

void mouse2top_right_corner(keyrecord_t *record) {
     mouse_jump(w, -h, 5);
     tap_code(KC_ESC);
}

void  mouse2bottom_left_corner(keyrecord_t *record) {
     mouse_jump(-w, h, 5);
     tap_code(KC_ESC);
}

void  mouse2bottom_right_corner(keyrecord_t *record) {
     mouse_jump(w, h, 5);
     tap_code(KC_ESC);
}

static uint16_t mouse_timer;
static int mouse_flag = 0;
static int mouse_count = 0;

void matrix_scan_user (void) {
     int interval;
     int8_t ls = 3; // long-side
     int8_t ss = 1; // short-side

     if (mouse_flag > 0) {
          if (mouse_count < 1) {
               interval = 2;
          } else if (mouse_count < 5) {
               interval = 40;
          } else if (mouse_count < 10) {
               interval = 10;
          } else {
               interval = 10;
               ls = 6;
               ss = 2;
          }
          if(timer_elapsed(mouse_timer) > interval) {
               switch(mouse_flag) {
               case  A_MLUL:
                    mouse_move(-ls, -ss);
                    break;

               case  A_MULU:
                    mouse_move(-ss, -ls);
                    break;

               case A_MURU:
                    mouse_move(ss, -ls);
                    break;

               case A_MRUR:
                    mouse_move(ls, -ss);
                    break;

               case A_MLDL:
                    mouse_move(-ls, ss);
                    break;

               case A_MDLD:
                    mouse_move(-ss, ls);
                    break;

               case A_MDRD:
                    mouse_move(ss, ls);
                    break;

               case A_MRDR:
                    mouse_move(ls, ss);
                    break;
               }
               //mousekey_clear();
               mouse_count++;
               mouse_timer = timer_read();  //restart timer
          }
     }
}

void settimer_mousemove(keyrecord_t *record,  int flag){
     if (record->event.pressed) {
          mouse_timer = timer_read();  //start timer
          mouse_flag = flag;
     } else {
          mouse_flag = 0;
          mouse_count = 0;
     }

}


const macro_t *action_get_macro(keyrecord_t *record, uint8_t id, uint8_t opt)
{
     const int8_t row = 20;
     const int8_t col = 54;

     switch(id) {
     case 0:
          if (record->event.pressed) {
            register_code(KC_RSFT);
          } else {
            unregister_code(KC_RSFT);
          }
          break;


     case A_MLUL:
          settimer_mousemove(record,  A_MLUL);
          break;

     case A_MUL:
          mousemove_dgnl(record, KC_MS_UP, KC_MS_LEFT, 1);
          break;

     case A_MULU:
          settimer_mousemove(record,  A_MULU);
          break;

     case A_MURU:
          settimer_mousemove(record,  A_MURU);
          break;

     case A_MUR:
          mousemove_dgnl(record, KC_MS_UP, KC_MS_RIGHT, 1);
          break;

     case A_MRUR:
          settimer_mousemove(record,  A_MRUR);
          break;

     case A_MLDL:
          settimer_mousemove(record,  A_MLDL);
          break;

     case A_MDL:

          mousemove_dgnl(record, KC_MS_DOWN, KC_MS_LEFT, 1);
          break;

     case A_MDLD:
          settimer_mousemove(record,  A_MDLD);
          break;

     case A_MDRD:
          settimer_mousemove(record,  A_MDRD);
          break;

     case A_MDR:
          mousemove_dgnl(record, KC_MS_DOWN, KC_MS_RIGHT, 1);
          break;

     case A_MRDR:
          settimer_mousemove(record,  A_MRDR);
          break;


     case A_MTLC:
          mouse2top_left_corner(record);
          break;

     case A_MTRC:
          mouse2top_right_corner(record);
          break;

     case A_MBLC:
          mouse2bottom_left_corner(record);
          break;

     case A_MBRC:
          mouse2bottom_right_corner(record);
          break;

     case  A_MR1C1:
           mouse2top_left_corner(record);
           mouse_jump(col, row, 3);
           break;

      case  A_MR1C2:
           mouse2top_left_corner(record);
           mouse_jump(col * 2, row, 3);
           break;

     case  A_MR1C3:
           mouse2top_right_corner(record);
           mouse_jump(-col * 2, row, 3);
           break;

     case  A_MR1C4:
           mouse2top_right_corner(record);
           mouse_jump(-col, row, 3);
           break;


      case  A_MLE:
           mouse2top_left_corner(record);
           mouse_jump(0, row *4, 3);
           break;

      case  A_MR2C1:
           mouse2top_left_corner(record);
           mouse_jump(col, row *4, 3);
           break;

      case  A_MR2C2:
           mouse2top_left_corner(record);
           mouse_jump(col * 2, row *4, 3);
           break;

     case  A_MR2C3:
           mouse2top_right_corner(record);
           mouse_jump(-col * 2, row *4, 3);
           break;

     case  A_MR2C4:
           mouse2top_right_corner(record);
           mouse_jump(-col, row *4, 3);
           break;

     case  A_MRE:
           mouse2top_right_corner(record);
           mouse_jump(0, row *4, 3);
           break;

      case  A_MR3C1:
           mouse2bottom_left_corner(record);
           mouse_jump(col, -row, 3);
           break;

      case  A_MR3C2:
           mouse2bottom_left_corner(record);
           mouse_jump(col * 2, -row, 3);
           break;

     case  A_MR3C3:
           mouse2bottom_right_corner(record);
           mouse_jump(-col * 2, -row, 3);
           break;

     case  A_MR3C4:
           mouse2bottom_right_corner(record);
           mouse_jump(-col, -row, 3);
           break;

      }
      return MACRO_NONE;
};



void persistent_default_layer_set(uint16_t default_layer) {
  eeconfig_update_default_layer(default_layer);
  default_layer_set(default_layer);
}
